precision highp int;
precision highp float;
 #include <OgreUnifiedShader.h>
    varying vec2 globalHeatmapUV;
varying vec2 personalHeatmapUV;
uniform sampler2D _GlobalHeatmapTex;
uniform sampler2D _GlobalHeatmapColorMapTex;
uniform sampler2D _PersonalHeatmapTex;
uniform sampler2D _PersonalHeatmapColorMapTex;
uniform int globalHeatmapEnabled;
uniform int personalHeatmapEnabled;
uniform float globalHeatmapOpacity;
uniform float personalHeatmapOpacity;
const float kHalfColorMapPixel = 0.5 / 255.0;
vec4 applyColorMap(sampler2D colorMap, float intensity)
{
    vec2 coords = vec2(mix(kHalfColorMapPixel, 1.0 - kHalfColorMapPixel, intensity), 0.5);
    vec4 color = texture2D(colorMap, coords);
    color.rgb *= color.a;
    return color;
}
vec4 alphaBlend(vec4 bgColor, vec4 fgColor)
{
    return fgColor + (1.0 - fgColor.a) * bgColor;
}
void main()
{
    vec4 blendedColor = vec4(0.0);
    if (globalHeatmapEnabled != 0)
    {
        float globalHeatmapIntensity = texture2D(_GlobalHeatmapTex, globalHeatmapUV).r;
        vec4 globalHeatmapColor = applyColorMap(_GlobalHeatmapColorMapTex, globalHeatmapIntensity);
        globalHeatmapColor *= globalHeatmapOpacity;
        blendedColor = alphaBlend(blendedColor, globalHeatmapColor);
    }
    if (personalHeatmapEnabled != 0)
    {
        float personalHeatmapIntensity = texture2D(_PersonalHeatmapTex, personalHeatmapUV).r;
        vec4 personalHeatmapColor =
            applyColorMap(_PersonalHeatmapColorMapTex, personalHeatmapIntensity);
        personalHeatmapColor *= personalHeatmapOpacity;
        blendedColor = alphaBlend(blendedColor, personalHeatmapColor);
    }
    gl_FragColor = blendedColor;
}
